@@ -31,6 +31,11 @@ from ibus import modifier
 import jastring
 from segment import unichar_half_to_full
 
+try:
+    from gtk import clipboard_get
+except ImportError:
+    clipboard_get = lambda a : None
+
 sys.path.append(path.join(os.getenv('IBUS_ANTHY_PKGDATADIR'), 'setup'))
 from anthyprefs import AnthyPrefs
 
@@ -56,7 +61,10 @@ CONV_MODE_LATIN_3, \
 CONV_MODE_WIDE_LATIN_0, \
 CONV_MODE_WIDE_LATIN_1, \
 CONV_MODE_WIDE_LATIN_2, \
-CONV_MODE_WIDE_LATIN_3 = range(13)
+CONV_MODE_WIDE_LATIN_3, \
+CONV_MODE_PREDICTION = range(14)
+
+CLIPBOARD_RECONVERT = range(1)
 
 KP_Table = {}
 for k, v in zip(['KP_Add', 'KP_Decimal', 'KP_Divide', 'KP_Enter', 'KP_Equal',
@@ -181,6 +189,14 @@ class Engine(ibus.EngineBase):
 
         return anthy_props
 
+    def __get_clipboard(self, clipboard, text, data):
+        clipboard_text = clipboard.wait_for_text ()
+
+        if data == CLIPBOARD_RECONVERT:
+            self.__update_reconvert(clipboard_text)
+
+        return clipboard_text
+
     def update_preedit(self, string, attrs, cursor_pos, visible):
         self.update_preedit_text(ibus.Text(string, attrs), cursor_pos, visible)
 
@@ -217,7 +233,8 @@ class Engine(ibus.EngineBase):
 
     def cursor_up(self):
         # only process cursor down in convert mode
-        if self.__convert_mode != CONV_MODE_ANTHY:
+        # if self.__convert_mode != CONV_MODE_ANTHY:
+        if self.__convert_mode != CONV_MODE_ANTHY and self.__convert_mode != CONV_MODE_PREDICTION:
             return False
 
         if not self.__lookup_table.cursor_up():
@@ -231,7 +248,8 @@ class Engine(ibus.EngineBase):
 
     def cursor_down(self):
         # only process cursor down in convert mode
-        if self.__convert_mode != CONV_MODE_ANTHY:
+        # if self.__convert_mode != CONV_MODE_ANTHY:
+        if self.__convert_mode != CONV_MODE_ANTHY and self.__convert_mode != CONV_MODE_PREDICTION:
             return False
 
         if not self.__lookup_table.cursor_down():
@@ -243,6 +261,13 @@ class Engine(ibus.EngineBase):
         self.__invalidate()
         return True
 
+    def candidate_clicked(self, index, button, state):
+        if index == 9:
+            keyval = keysyms._0
+        else:
+            keyval = keysyms._1 + index
+        self.__on_key_number(keyval)
+
     def __commit_string(self, text):
         self.__reset()
         self.commit_text(ibus.Text(text))
@@ -262,14 +287,22 @@ class Engine(ibus.EngineBase):
         self.__invalidate()
         return True
 
-    def process_key_event(self, keyval, state):
-        try:
-#            return self.process_key_event_internal(keyval, state)
-            return self.process_key_event_internal2(keyval, state)
-        except:
-            import traceback
-            traceback.print_exc()
-            return False
+    if ibus.get_version() >= '1.2.0':
+        def process_key_event(self, keyval, keycode, state):
+            try:
+                return self.process_key_event_internal2(keyval, keycode, state)
+            except:
+                import traceback
+                traceback.print_exc()
+                return False
+    else:
+        def process_key_event(self, keyval, state):
+            try:
+                return self.process_key_event_internal2(keyval, 0, state)
+            except:
+                import traceback
+                traceback.print_exc()
+                return False
 
     def process_key_event_internal(self, keyval, state):
         is_press = (state & modifier.RELEASE_MASK) == 0
@@ -472,6 +505,19 @@ class Engine(ibus.EngineBase):
         self.__end_anthy_convert()
 
     def __fill_lookup_table(self):
+        if self.__convert_mode == CONV_MODE_PREDICTION:
+            seg_stat = anthy.anthy_prediction_stat()
+            self.__context.get_prediction_stat(seg_stat)
+
+            # fill lookup_table
+            self.__lookup_table.clean()
+            for i in xrange(0, seg_stat.nr_prediction):
+                buf = self.__context.get_prediction(i)
+                candidate = unicode(buf, "utf-8")
+                self.__lookup_table.append_candidate(ibus.Text(candidate))
+
+            return
+
         # get segment stat
         seg_stat = anthy.anthy_segment_stat()
         self.__context.get_segment_stat(self.__cursor_pos, seg_stat)
@@ -519,7 +565,8 @@ class Engine(ibus.EngineBase):
             self.__lookup_table_visible)
 
     def __update_convert_chars(self):
-        if self.__convert_mode == CONV_MODE_ANTHY:
+#        if self.__convert_mode == CONV_MODE_ANTHY:
+        if self.__convert_mode == CONV_MODE_ANTHY or self.__convert_mode == CONV_MODE_PREDICTION:
             self.__update_anthy_convert_chars()
             return
         if self.__convert_mode == CONV_MODE_HIRAGANA:
@@ -575,12 +622,10 @@ class Engine(ibus.EngineBase):
     def __update_anthy_convert_chars(self):
         self.__convert_chars = u""
         pos = 0
-        i = 0
-        for seg_index, text in self.__segments:
+        for i, (seg_index, text) in enumerate(self.__segments):
             self.__convert_chars += text
             if i < self.__cursor_pos:
                 pos += len(text)
-            i += 1
         attrs = ibus.AttrList()
         attrs.append(ibus.AttributeUnderline(
             ibus.ATTR_UNDERLINE_SINGLE, 0, len(self.__convert_chars)))
@@ -611,10 +656,12 @@ class Engine(ibus.EngineBase):
             text, cursor = self.__get_preedit(True)
             self.__commit_string(text)
         elif self.__convert_mode == CONV_MODE_ANTHY:
-            i = 0
-            for seg_index, text in self.__segments:
+            for i, (seg_index, text) in enumerate(self.__segments):
                 self.__context.commit_segment(i, seg_index)
             self.__commit_string(self.__convert_chars)
+        elif self.__convert_mode == CONV_MODE_PREDICTION:
+            self.__context.commit_prediction(self.__segments[0][0])
+            self.__commit_string(self.__convert_chars)
         else:
             self.__commit_string(self.__convert_chars)
 
@@ -870,8 +917,7 @@ class Engine(ibus.EngineBase):
 
         # Input Japanese
         if self.__convert_mode == CONV_MODE_ANTHY:
-            i = 0
-            for seg_index, text in self.__segments:
+            for i, (seg_index, text) in enumerate(self.__segments):
                 self.__context.commit_segment(i, seg_index)
             self.__commit_string(self.__convert_chars)
         elif self.__convert_mode != CONV_MODE_OFF:
@@ -888,12 +934,7 @@ class Engine(ibus.EngineBase):
         if not cls.__prefs:
             cls.__prefs = AnthyPrefs(bus)
 
-        keybind = {}
-        for k in cls.__prefs.keys('shortcut/default'):
-            cmd = '_Engine__cmd_' + k
-            for s in cls.__prefs.get_value('shortcut/default', k):
-                keybind.setdefault(cls._s_to_key(s), []).append(cmd)
-        cls.__keybind = keybind
+        cls.__keybind = cls._mk_keybind()
 
         jastring.JaString._prefs = cls.__prefs
 
@@ -901,22 +942,45 @@ class Engine(ibus.EngineBase):
     def CONFIG_VALUE_CHANGED(cls, bus, section, name, value):
         print 'VALUE_CHAMGED =', section, name, value
         section = section[len(cls.__prefs._prefix) + 1:]
-        if section.startswith('shortcut/'):
+        if section == cls._get_shortcut_type():
             cmd = '_Engine__cmd_' + name
-            old = cls.__prefs.get_value('shortcut/default', name)
+            sec = cls._get_shortcut_type()
+            old = cls.__prefs.get_value(sec, name)
             value = value if value != [''] else []
             for s in set(old).difference(value):
                 cls.__keybind.get(cls._s_to_key(s), []).remove(cmd)
 
-            keys = cls.__prefs.keys('shortcut/default')
+            keys = cls.__prefs.keys(sec)
             for s in set(value).difference(old):
                 cls.__keybind.setdefault(cls._s_to_key(s), []).append(cmd)
                 cls.__keybind.get(cls._s_to_key(s)).sort(
                     lambda a, b: cmp(keys.index(a[13:]), keys.index(b[13:])))
 
-            cls.__prefs.set_value('shortcut/default', name, value)
+            cls.__prefs.set_value(sec, name, value)
         elif section == 'common':
             cls.__prefs.set_value(section, name, value)
+            if name == 'shortcut_type':
+                cls.__keybind = cls._mk_keybind()
+        else:
+            cls.__prefs.set_value(sec, name, value)
+
+    @classmethod
+    def _mk_keybind(cls):
+        keybind = {}
+        sec = cls._get_shortcut_type()
+        for k in cls.__prefs.keys(sec):
+            cmd = '_Engine__cmd_' + k
+            for s in cls.__prefs.get_value(sec, k):
+                keybind.setdefault(cls._s_to_key(s), []).append(cmd)
+        return keybind
+
+    @classmethod
+    def _get_shortcut_type(cls):
+        try:
+            t = 'shortcut/' + cls.__prefs.get_value('common', 'shortcut_type')
+        except:
+            t = 'shortcut/default'
+        return t
 
     @classmethod
     def _s_to_key(cls, s):
@@ -937,7 +1001,7 @@ class Engine(ibus.EngineBase):
 
         return repr([int(state), int(keyval)])
 
-    def process_key_event_internal2(self, keyval, state):
+    def process_key_event_internal2(self, keyval, keycode, state):
 
         is_press = (state & modifier.RELEASE_MASK) == 0
 
@@ -967,6 +1031,11 @@ class Engine(ibus.EngineBase):
 
         if (keysyms.exclam <= keyval <= keysyms.asciitilde or
             keyval == keysyms.yen):
+            if self.__typing_mode == jastring.TYPING_MODE_KANA:
+                if keyval == keysyms._0 and state == modifier.SHIFT_MASK:
+                    keyval = keysyms.asciitilde
+                elif keyval == keysyms.backslash and keycode in [132-8, 133-8]:
+                    keyval = keysyms.yen
             ret = self.__on_key_common(keyval)
             if (unichr(keyval) in u',.' and
                 self.__prefs.get_value('common', 'behivior_on_period')):
@@ -977,9 +1046,31 @@ class Engine(ibus.EngineBase):
                 return True
             return False
 
-    #mode_keys
+    def _chk_mode(self, mode):
+        if '0' in mode and self.__preedit_ja_string.is_empty():
+            return True
+
+        if self.__convert_mode == CONV_MODE_OFF:
+            if '1' in mode and not self.__preedit_ja_string.is_empty():
+                return True
+        elif self.__convert_mode == CONV_MODE_ANTHY:
+            if '2' in mode and not self.__lookup_table_visible:
+                return True
+        elif self.__convert_mode == CONV_MODE_PREDICTION:
+            if '3' in mode and not self.__lookup_table_visible:
+                return True
+        else:
+            if '4' in mode:
+                return True
+
+        if '5' in mode and self.__lookup_table_visible:
+            return True
+
+        return False
+
+    #mod_keys
     def __set_input_mode(self, mode):
-        if not self.__preedit_ja_string.is_empty():
+        if not self._chk_mode('0'):
             return False
 
         self.__input_mode_activate(mode, ibus.PROP_STATE_CHECKED)
@@ -1027,11 +1118,11 @@ class Engine(ibus.EngineBase):
     def __cmd_half_katakana(self, keyval, state):
         return self.__set_input_mode(u'InputMode.HalfWidthKatakana')
 
-    def __cmd_cancel_pseudo_ascii_mode_key(self, keyval, state):
-        pass
+#    def __cmd_cancel_pseudo_ascii_mode_key(self, keyval, state):
+#        pass
 
     def __cmd_circle_typing_method(self, keyval, state):
-        if not self.__preedit_ja_string.is_empty():
+        if not self._chk_mode('0'):
             return False
 
         modes = {
@@ -1058,12 +1149,18 @@ class Engine(ibus.EngineBase):
             return self.__cmd_insert_wide_space(keyval, state)
 
     def __cmd_insert_half_space(self, keyval, state):
+        if not self._chk_mode('0'):
+            return False
+
         if not self.__preedit_ja_string.is_empty():
             return False
         self.__commit_string(unichr(keysyms.space))
         return True
 
     def __cmd_insert_wide_space(self, keyval, state):
+        if not self._chk_mode('0'):
+            return False
+
         if not self.__preedit_ja_string.is_empty():
             return False
         char = unichr(keysyms.space)
@@ -1074,31 +1171,65 @@ class Engine(ibus.EngineBase):
         return True
 
     def __cmd_backspace(self, keyval, state):
+        if not self._chk_mode('12345'):
+            return False
+
         return self.__on_key_back_space()
 
     def __cmd_delete(self, keyval, state):
+        if not self._chk_mode('12345'):
+            return False
+
         return self.__on_key_delete()
 
     def __cmd_commit(self, keyval, state):
+        if not self._chk_mode('12345'):
+            return False
+
         return self.__on_key_return()
 
     def __cmd_convert(self, keyval, state):
-        if self.__preedit_ja_string.is_empty() or \
-                self.__input_mode != INPUT_MODE_HIRAGANA:
+        if not self._chk_mode('14'):
             return False
-        if self.__convert_mode != CONV_MODE_ANTHY:
-            self.__begin_anthy_convert()
-            self.__invalidate()
-        elif self.__convert_mode == CONV_MODE_ANTHY:
-            self.__lookup_table_visible = True
-            self.cursor_down()
+
+        self.__begin_anthy_convert()
+        self.__invalidate()
+
         return True
 
     def __cmd_predict(self, keyval, state):
-        pass
+        if not self._chk_mode('14'):
+            return False
+
+        text, cursor = self.__preedit_ja_string.get_hiragana(True)
+
+        self.__context.set_prediction_string(text.encode("utf8"))
+        ps = anthy.anthy_prediction_stat()
+        self.__context.get_prediction_stat(ps)
+
+#        for i in range(ps.nr_prediction):
+#            print self.__context.get_prediction(i)
+
+        buf = self.__context.get_prediction(0)
+        if not buf:
+            return False
+
+        text = unicode(buf, "utf-8")
+        self.__segments.append((0, text))
+
+        self.__convert_mode = CONV_MODE_PREDICTION
+        self.__cursor_pos = 0
+        self.__fill_lookup_table()
+        self.__lookup_table_visible = False
+        self.__invalidate()
+
+        return True
 
     def __cmd_cancel(self, keyval, state):
-        if self.__preedit_ja_string.is_empty():
+        return self.__cmd_cancel_all(keyval, state)
+
+    def __cmd_cancel_all(self, keyval, state):
+        if not self._chk_mode('12345'):
             return False
 
         if self.__convert_mode == CONV_MODE_OFF:
@@ -1108,21 +1239,51 @@ class Engine(ibus.EngineBase):
             self.__invalidate()
             return True
 
-    def __cmd_cancel_all(self, keyval, state):
-        return self.__cmd_cancel(keyval, state)
-
     def __cmd_reconvert(self, keyval, state):
-        pass
+        if not self.__preedit_ja_string.is_empty():
+            # if user has inputed some chars
+            return False
+
+        # Use gtk.Clipboard.request_text() instead of
+        # gtk.Clipboard.wait_for_text() because DBus is timed out.
+        clipboard = clipboard_get ("PRIMARY")
+        if clipboard:
+            clipboard.request_text (self.__get_clipboard, CLIPBOARD_RECONVERT)
 
-    def __cmd_do_nothing(self, keyval, state):
         return True
 
+    def __update_reconvert(self, clipboard_text):
+        if clipboard_text == None:
+            return False
+
+        self.__convert_chars = unicode (clipboard_text, "utf-8")
+        for i in xrange(0, len(self.__convert_chars)):
+            keyval = self.__convert_chars[i]
+            self.__preedit_ja_string.insert(unichr(ord (keyval)))
+
+        self.__context.set_string(self.__convert_chars.encode("utf-8"))
+        conv_stat = anthy.anthy_conv_stat()
+        self.__context.get_stat(conv_stat)
+
+        for i in xrange(0, conv_stat.nr_segment):
+            buf = self.__context.get_segment(i, 0)
+            text = unicode(buf, "utf-8")
+            self.__segments.append((0, text))
+
+        self.__convert_mode = CONV_MODE_ANTHY
+        self.__cursor_pos = 0
+        self.__fill_lookup_table()
+        self.__lookup_table_visible = False
+        self.__invalidate()
+
+        return True
+
+#    def __cmd_do_nothing(self, keyval, state):
+#        return True
+
     #caret_keys
     def __move_caret(self, i):
-        if self.__lookup_table_visible:
-            return False
-
-        if self.__preedit_ja_string.is_empty():
+        if not self._chk_mode('1'):
             return False
 
         if self.__convert_mode == CONV_MODE_OFF:
@@ -1149,14 +1310,9 @@ class Engine(ibus.EngineBase):
 
     #segments_keys
     def __select_segment(self, i):
-        if self.__preedit_ja_string.is_empty():
+        if not self._chk_mode('25'):
             return False
 
-        if self.__convert_mode == CONV_MODE_OFF:
-            return False
-        elif self.__convert_mode != CONV_MODE_ANTHY:
-            return True
-
         pos = 0 if i == 0 else \
               self.__cursor_pos + i if i in [-1, 1] else \
               len(self.__segments) - 1
@@ -1170,15 +1326,9 @@ class Engine(ibus.EngineBase):
         return True
 
     def __cmd_select_first_segment(self, keyval, state):
-        if self.__lookup_table_visible:
-            return False
-
         return self.__select_segment(0)
 
     def __cmd_select_last_segment(self, keyval, state):
-        if self.__lookup_table_visible:
-            return False
-
         return self.__select_segment(2)
 
     def __cmd_select_next_segment(self, keyval, state):
@@ -1188,24 +1338,74 @@ class Engine(ibus.EngineBase):
         return self.__select_segment(-1)
 
     def __cmd_shrink_segment(self, keyval, state):
+        if not self._chk_mode('25'):
+            return False
+
         if self.__convert_mode == CONV_MODE_ANTHY:
             self.__shrink_segment(-1)
             return True
 
     def __cmd_expand_segment(self, keyval, state):
+        if not self._chk_mode('25'):
+            return False
+
         if self.__convert_mode == CONV_MODE_ANTHY:
             self.__shrink_segment(1)
             return True
 
+    def __commit_nth_segment(self, commit_index, keyval, state):
+
+        if commit_index >= len(self.__segments):
+            return False
+
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            for i in xrange(0, commit_index + 1):
+                (seg_index, text) = self.__segments[i]
+                self.commit_text(ibus.Text(text))
+
+            text, cursor = self.__get_preedit()
+            commit_length = 0
+            for i in xrange(0, commit_index + 1):
+                buf = self.__context.get_segment(i, -1)
+                commit_length += len(unicode(buf, "utf-8"))
+            self.__preedit_ja_string.move_cursor(commit_length - cursor)
+            for i in xrange(0, commit_length):
+                self.__preedit_ja_string.remove_before()
+            self.__preedit_ja_string.move_cursor(cursor - commit_length)
+
+            del self.__segments[0:commit_index + 1]
+
+        if len(self.__segments) == 0:
+            self.__reset()
+        else:
+            if self.__cursor_pos > commit_index:
+                self.__cursor_pos -= (commit_index + 1)
+            else:
+                self.__cursor_pos = 0
+            (seg_index, text) = self.__segments[self.__cursor_pos]
+            self.__convert_chars = text
+            self.__context.set_string(text.encode ("utf-8"))
+
+        self.__lookup_table.clean()
+        self.__lookup_table.show_cursor (False)
+        self.__lookup_table_visible = False
+        self.update_aux_string(u"", ibus.AttrList(),
+            self.__lookup_table_visible)
+        self.__fill_lookup_table()
+        self.__invalidate()
+        self.__update_input_chars()
+
+        return True
+
     def __cmd_commit_first_segment(self, keyval, state):
-        pass
+        return self.__commit_nth_segment(0, keyval, state)
 
     def __cmd_commit_selected_segment(self, keyval, state):
-        pass
+        return self.__commit_nth_segment(self.__cursor_pos, keyval, state)
 
     #candidates_keys
     def __select_candidate(self, pos):
-        if not self.__lookup_table_visible:
+        if not self._chk_mode('5'):
             return False
 
         if not self.__lookup_table.set_cursor_pos_in_current_page(pos):
@@ -1224,79 +1424,152 @@ class Engine(ibus.EngineBase):
         return self.__select_candidate(self.__lookup_table.get_page_size() - 1)
 
     def __cmd_select_next_candidate(self, keyval, state):
+        if not self._chk_mode('235'):
+            return False
+
         return self.__on_key_down()
 
     def __cmd_select_prev_candidate(self, keyval, state):
+        if not self._chk_mode('235'):
+            return False
+
         return self.__on_key_up()
 
     def __cmd_candidates_page_up(self, keyval, state):
+        if not self._chk_mode('5'):
+            return False
+
         return self.__on_key_page_up()
 
     def __cmd_candidates_page_down(self, keyval, state):
+        if not self._chk_mode('5'):
+            return False
+
         return self.__on_key_page_down()
 
     #direct_select_keys
-    def __cmd_select_candidates_1(self, keyval, state):
+    def __select_candidates(self, keyval):
+        if not self._chk_mode('5'):
+            return False
+
         return self.__on_key_number(keyval)
 
+    def __cmd_select_candidates_1(self, keyval, state):
+        return self.__select_candidates(keyval)
+
     def __cmd_select_candidates_2(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_3(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_4(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_5(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_6(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_7(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_8(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_9(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     def __cmd_select_candidates_0(self, keyval, state):
-        return self.__on_key_number(keyval)
+        return self.__select_candidates(keyval)
 
     #convert_keys
     def __cmd_convert_to_char_type_forward(self, keyval, state):
-        pass
+        if self.__convert_mode == CONV_MODE_KATAKANA:
+            return self.__cmd_convert_to_half(keyval, state)
+        elif self.__convert_mode == CONV_MODE_HALF_WIDTH_KATAKANA:
+            return self.__cmd_convert_to_latin(keyval, state)
+        elif CONV_MODE_LATIN_0 <= self.__convert_mode <= CONV_MODE_LATIN_3:
+            return self.__cmd_convert_to_wide_latin(keyval, state)
+        elif (CONV_MODE_WIDE_LATIN_0 <= self.__convert_mode
+                                     <= CONV_MODE_WIDE_LATIN_3):
+            return self.__cmd_convert_to_hiragana(keyval, state)
+        else:
+            return self.__cmd_convert_to_katakana(keyval, state)
 
     def __cmd_convert_to_char_type_backward(self, keyval, state):
-        pass
+        if self.__convert_mode == CONV_MODE_KATAKANA:
+            return self.__cmd_convert_to_hiragana(keyval, state)
+        elif self.__convert_mode == CONV_MODE_HALF_WIDTH_KATAKANA:
+            return self.__cmd_convert_to_katakana(keyval, state)
+        elif CONV_MODE_LATIN_0 <= self.__convert_mode <= CONV_MODE_LATIN_3:
+            return self.__cmd_convert_to_half(keyval, state)
+        elif (CONV_MODE_WIDE_LATIN_0 <= self.__convert_mode
+                                     <= CONV_MODE_WIDE_LATIN_3):
+            return self.__cmd_convert_to_latin(keyval, state)
+        else:
+            return self.__cmd_convert_to_wide_latin(keyval, state)
 
     def __cmd_convert_to_hiragana(self, keyval, state):
+        if not self._chk_mode('1234'):
+            return False
+
         return self.__on_key_conv(0)
 
     def __cmd_convert_to_katakana(self, keyval, state):
+        if not self._chk_mode('1234'):
+            return False
+
         return self.__on_key_conv(1)
 
     def __cmd_convert_to_half(self, keyval, state):
+        if not self._chk_mode('1234'):
+            return False
+
         return self.__on_key_conv(2)
 
     def __cmd_convert_to_half_katakana(self, keyval, state):
+        if not self._chk_mode('1234'):
+            return False
+
         return self.__on_key_conv(2)
 
     def __cmd_convert_to_wide_latin(self, keyval, state):
+        if not self._chk_mode('1234'):
+            return False
+
         return self.__on_key_conv(3)
 
     def __cmd_convert_to_latin(self, keyval, state):
+        if not self._chk_mode('1234'):
+            return False
+
         return self.__on_key_conv(4)
 
     #dictonary_keys
     def __cmd_dict_admin(self, keyval, state):
-        pass
+        if not self._chk_mode('0'):
+            return False
+
+        path = self.__prefs.get_value('common', 'dict_admin_command')
+        os.spawnl(os.P_NOWAIT, *path)
+        return True
 
     def __cmd_add_word(self, keyval, state):
-        pass
+        if not self._chk_mode('0'):
+            return False
+
+        path = self.__prefs.get_value('common', 'add_word_command')
+        os.spawnl(os.P_NOWAIT, *path)
+        return True
+
+    def __cmd_start_setup(self, keyval, state):
+        if not self._chk_mode('0'):
+            return False
+
+        self.__start_setup()
+        return True
 
     def __start_setup(self):
         if Engine.__setup_pid != 0:
