@@ -37,6 +37,10 @@ romaji_typing_rule = {
     u"lu" : u"ぅ",
     u"le" : u"ぇ",
     u"lo" : u"ぉ",
+#    u"xka" : u"ゕ",
+    u"xka" : u"ヵ",
+#    u"xke" : u"ゖ",
+    u"xke" : u"ヶ",
     u"wi" : u"うぃ",
     u"we" : u"うぇ",
     u"wha" : u"うぁ",
@@ -133,6 +137,9 @@ romaji_typing_rule = {
     u"dyu" : u"ぢゅ",
     u"dye" : u"ぢぇ",
     u"dyo" : u"ぢょ",
+    u"tsa" : u"つぁ",
+    u"tse" : u"つぇ",
+    u"tso" : u"つぉ",
     u"tha" : u"てゃ",
     u"thi" : u"てぃ",
     u"thu" : u"てゅ",
@@ -225,7 +232,7 @@ romaji_typing_rule = {
     u"xwa" : u"ゎ",
     u"wa" : u"わ",
     u"wo" : u"を",
-    # u"n'" : u"ん",
+    u"n'" : u"ん",
     u"nn" : u"ん",
     u"wyi" : u"ゐ",
     u"wye" : u"ゑ",
@@ -599,7 +606,9 @@ hiragana_katakana_table = {
     u"｜" : (u"｜", u"|"),
     u"\u2018"  : (u"\u2018", u"`"),
     u"＠" : (u"＠", u"@"),
+    u"゛" : (u"゛", u"ﾞ"),
     u"｛" : (u"｛", u"{"),
+    u"゜" : (u"゜", u"ﾟ"),
     u"「" : (u"「", u"｢"),
     u"＋" : (u"＋", u"+"),
     u"；" : (u"；", u";"),
@@ -610,6 +619,7 @@ hiragana_katakana_table = {
     u"＜" : (u"＜", u"<"),
     u"＞" : (u"＞", u">"),
     u"？" : (u"？", u"?"),
+    u"・" : (u"・", u"･"),
     u"／" : (u"／", u"/"),
     u"＿" : (u"＿", u"_"),
     u"￥" : (u"￥", u"¥"),
